/*
GoFans Client Copyright (C) 2010 Robert Grning (GoFans)
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under certain conditions. 
<http://www.gnu.org/licenses/>
*/
//class
function GameConverterXml(){

 this.Game = null;
 this.Xml = null;
 this.Message = null;
}

//public
GameConverterXml.prototype.Export = function(){  

 this.Message = null;
 this.Game.GoToCleanBoard();
 
 while(this.Game.GoToNext(0) != 209)
 {}
 
 this.Game.SetTerritory();
 
 this.Xml = this.private_WriteXml(this.Game); 
}

//private
GameConverterXml.prototype.private_WriteXml = function(game){
 
 var xml = "<?xml version=\"1.0\" encoding=\"utf-8\" ?>";

 xml += this.private_WriteGame(game);
 
 return xml;
}

//private
GameConverterXml.prototype.private_WriteGame = function(game){
 
 var xml = "<game>";
 var format = new Format();
 format.Version = "2.2";
 format.Publisher = "GoFans";

 xml += this.private_WriteFormat(format);
 xml += this.private_WriteHead(game);
 xml += this.private_WriteBody(game.Moves); 
 xml += "</game>";

 return xml;
}

//private
GameConverterXml.prototype.private_WriteFormat = function(format){
 
 var xml = "<format version=\""+format.Version+"\">";
 xml += "<publisher name=\""+format.Publisher+"\"/>"; 
 xml += "</format>";

 return xml;
}

//private
GameConverterXml.prototype.private_WriteHead = function(game){
 
 var xml = "<head>";
 xml += this.private_WriteRule(game.Rule);
 xml += this.private_WriteBoard(game.Board);
 xml += this.private_WritePlayers(game.Players);
 xml += this.private_WriteServer(game.Server);
 xml += this.private_WriteDate(game.Date);
 xml += this.private_WriteComputation(game.Computation);
 xml += "</head>";

 return xml;
}

//private
GameConverterXml.prototype.private_WritePlayers = function(players){
 
	var xml = "<players>";
	for (var i = 0; i < players.length; i++)
	{
		xml += "<player id=\""+players[i].Id+"\" color=\""+players[i].Color+"\" name=\""+players[i].Name+"\" strength=\""+players[i].Strength+"\" strengthClass=\""+players[i].StrengthClass+"\"";

		if(players[i].Handicap > 0)
		{
			xml += " handicap=\""+players[i].Handicap+"\"";
		}
		if(players[i].Komi > 0)
		{
			xml +=  " komi=\""+players[i].Komi+"\"";
		}
		
		if(this.Game.IsOneMoreMoveAllowed() == false)
		{
			xml += " prisoners=\""+players[i].Prisoners+"\" territory=\""+players[i].Territory+"\"";
			
			if(players[i].HasCapitulated == true)
			{
				xml += " hasCapitulated=\""+players[i].HasCapitulated+"\"";
			}
		}
		xml += "/>";
	} 
	xml += "</players>";

	return xml;
}

//private
GameConverterXml.prototype.private_WriteServer = function(server){
 
 return "<server address=\""+server+"\" />";
}

//private
GameConverterXml.prototype.private_WriteBoard = function(board){
 
 return "<board size=\""+board.Size+"\" />";
}

//private
GameConverterXml.prototype.private_WriteDate = function(date){
 
 return "<date dateTime=\""+date.toUTCString()+"\" />";
}

//private
GameConverterXml.prototype.private_WriteRule = function(rule){
 
	return "<rule name=\""+rule.Name+"\" />";
}

//private
GameConverterXml.prototype.private_WriteComputation = function(computation){
 
	return "<computation type=\""+computation.Type+"\"/>";
}

//private
GameConverterXml.prototype.private_WriteBody = function(moves){
 
 var xml = "<body>"; 
 xml += this.private_WriteMoves(moves);
 xml += "</body>";

 return xml;
}

//private
GameConverterXml.prototype.private_WriteMoves = function(moves){
 
 var xml = "";
 
 if(moves.length > 1)
 {
  xml = "<moves>";

  for(var i = 0; i < moves.length; i++)
  {
   xml += this.private_WriteMove(moves[i]);
  }

  xml += "</moves>";
 } 
 else if(moves.length == 1)
 {
  xml += this.private_WriteMove(moves[0]);
 }

 return xml;
}

//private
GameConverterXml.prototype.private_WriteMove = function(move){
 
	var xml = "<move playerId=\""+move.PlayerId+"\"";
 
	switch (move.Type)
	{
		case "positioning" :
		{
			xml += " x=\""+move.X+"\" y=\""+move.Y+"\" >"; 
			xml += this.private_WriteMoves(move.Moves);
			xml += this.private_WritePrisoners(move.Prisoners);
			xml += "</move>"; 
			break;
		}
		case "pass" :
		{
			xml += " type=\""+move.Type+"\" >"; 
			xml += this.private_WriteMoves(move.Moves);
			xml += this.private_WritePrisoners(move.Prisoners);
			xml += "</move>"; 
			break;
		}
		case "capitulation" :
		{
			xml += " type=\""+move.Type+"\" />"; 
			break;
		}
	} 

	return xml;
}

//private
GameConverterXml.prototype.private_WritePrisoners = function(prisoners){
 
 var xml = "";
 
 if(prisoners.length > 1)
 {
  xml = "<prisoners>";

  for(var i = 0; i < prisoners.length; i++)
  {
   xml += this.private_WritePrisoner(prisoners[i]);
  }

  xml += "</prisoners>";
 } 
 else if(prisoners.length == 1)
 {
  xml += this.private_WritePrisoner(prisoners[0]);
 }

 return xml;
}

//private
GameConverterXml.prototype.private_WritePrisoner = function(prisoner){
 
 return "<prisoner playerId=\""+prisoner.PlayerId+"\" x=\""+prisoner.X+"\" y=\""+prisoner.Y+"\" />";
}
 
//public
GameConverterXml.prototype.Import = function(){

 this.Message = null;
 this.Game = this.private_ReadGame(this.Xml); 
}
 
//private
GameConverterXml.prototype.private_ReadNode = function(node, name){
 
 var childNode = null;
 
 if(this.private_ReadNodes(node, name) != null)
 {
  childNode = this.private_ReadNodes(node, name)[0];
 }
 
 return childNode;
}
 
//private
GameConverterXml.prototype.private_ReadNodes = function(node, name){
 
 var childNodes = null;
 
 for (var i  = 0; i < node.childNodes.length; i++)
 {
  if(node.childNodes[i].tagName == name)  
  {
   if(childNodes == null)
   {
    childNodes = new Array();
   }
   childNodes[childNodes.length] = node.childNodes[i];
  }
 }
  
 return childNodes;
}

//private
GameConverterXml.prototype.private_ReadGame = function(node){
 
	var game = null;

	var format = null;

	var nodeGame = this.private_ReadNode(node, "game");
	var nodeFormat = this.private_ReadNode(nodeGame, "format");
	format = this.private_ReadFormat(nodeFormat);

	if (format.Publisher == "GoFans" && format.Version == "2.2") {

		var nodeHead = this.private_ReadNode(nodeGame, "head");
		var nodeRule = this.private_ReadNode(nodeHead, "rule");
		var nodeBody = this.private_ReadNode(nodeGame, "body");

		var ruleName = nodeRule.attributes.getNamedItem("name").value;

		switch (ruleName)
		{
			case "igo":
			{
				var game = new Igo();
				/*
				Fill here if there are more complex game types
				*/
				break;
			}
			case "iroirogote":
			{
				var game = new IroIroGote();
				/*
				Fill here if there are more complex game types
				*/
				break;
			}
		}
		
		game.Board = this.private_ReadBoard(nodeHead);
		game.Server = this.private_ReadServer(nodeHead);
		game.Date = this.private_ReadDate(nodeHead);
		game.Computation = this.private_ReadComputation(nodeHead);
		game.Players = this.private_ReadPlayers(nodeHead);
		game.Moves = this.private_ReadMoves(nodeBody);
		
		for(var i = 0; i < game.Players.length; i++)
		{
			game.Players[i].BitBoard = new BitBoard(game.Board.Size);
		}

		this.Message = 206;
	}
	else
	{
		this.Message = 407;
	}

	return game;
}

//private
GameConverterXml.prototype.private_ReadFormat = function(node){
 
 var format = new Format();

 var nodePublisher = this.private_ReadNode(node, "publisher");
 format.Version = node.attributes.getNamedItem("version").value;
 format.Publisher = nodePublisher.attributes.getNamedItem("name").value;
 
 return format;
}

//private
GameConverterXml.prototype.private_ReadDate = function(node){
 
 var date = null;

 var nodeDate = this.private_ReadNode(node, "date"); 
 date = new Date(nodeDate.attributes.getNamedItem("dateTime").value);
 
 return date;
}

//private
GameConverterXml.prototype.private_ReadServer = function(node){
 
 var server = null;

 var nodeServer = this.private_ReadNode(node, "server"); 
 server = nodeServer.attributes.getNamedItem("address").value;
 
 return server;
}

//private
GameConverterXml.prototype.private_ReadComputation = function(node){
 
 var computation = new Computation();

 var nodeComputation = this.private_ReadNode(node, "computation"); 
 computation.Type = nodeComputation.attributes.getNamedItem("type").value;
 
 return computation;
}

//private
GameConverterXml.prototype.private_ReadBoard = function(node){
 
 var board = new Board();

 var nodeBoard = this.private_ReadNode(node, "board"); 
 board.Size = parseInt(nodeBoard.attributes.getNamedItem("size").value);
 
 return board;
}

//private
GameConverterXml.prototype.private_ReadPlayers = function(node){
 
 var players = new Array();

 var nodePlayers = this.private_ReadNode(node, "players"); 
 var nodesPlayer = this.private_ReadNodes(nodePlayers, "player");
 
 for (var i = 0; i < nodesPlayer.length; i++)
 {
  players[players.length] = this.private_ReadPlayer(nodePlayers, i);
 }
 
 return players;
}

//private
GameConverterXml.prototype.private_ReadPlayer = function(node, index){
 
	var player = new Player();

	var nodePlayer = this.private_ReadNodes(node, "player")[index]; 
	player.Id = parseInt(nodePlayer.attributes.getNamedItem("id").value);
	player.Color = nodePlayer.attributes.getNamedItem("color").value;
	player.Name = nodePlayer.attributes.getNamedItem("name").value;
	player.Strength = parseInt(nodePlayer.attributes.getNamedItem("strength").value);
	player.StrengthClass = nodePlayer.attributes.getNamedItem("strengthClass").value;
	if(nodePlayer.attributes.getNamedItem("handicap") != null)
	{
		player.Handicap = parseInt(nodePlayer.attributes.getNamedItem("handicap").value);
	}
	if(nodePlayer.attributes.getNamedItem("komi") != null)
	{
		player.Komi = parseFloat(nodePlayer.attributes.getNamedItem("komi").value);
	}

	return player;
}

//private
GameConverterXml.prototype.private_ReadMoves = function(node){

 var moves = null;

 var nodeMoves = this.private_ReadNode(node, "moves"); 
 if(nodeMoves != null)
 {
  moves = new Array();
  var nodesMove = this.private_ReadNodes(nodeMoves, "move");
  for (var i = 0; i < nodesMove.length; i++)
  {
   moves[moves.length] = this.private_ReadMove(nodeMoves, i);
  }
 }
 else
 {
  if(this.private_ReadNode(node, "move") != null)
  {
   moves = new Array();
   moves[moves.length] = this.private_ReadMove(node, 0);
  }
 }
  
 return moves;
}

//private
GameConverterXml.prototype.private_ReadMove = function(node, index){
 
	var move = null;

	if(this.private_ReadNodes(node, "move") != null)
	{
		var nodeMove = this.private_ReadNodes(node, "move")[index];		
		var move = null;
		var moveType = "positioning";
		
		if(nodeMove.attributes.getNamedItem("type") != null)
		{
			moveType = nodeMove.attributes.getNamedItem("type").value;
		}
		switch (moveType)
		{
			case "positioning" :
			{
				move = new Positioning();
				move.X = parseInt(nodeMove.attributes.getNamedItem("x").value);
				move.Y = parseInt(nodeMove.attributes.getNamedItem("y").value);
				move.PlayerId = parseInt(nodeMove.attributes.getNamedItem("playerId").value);
				if(this.private_ReadNode(nodeMove, "moves") != null || this.private_ReadNodes(nodeMove, "move") != null)
				{
					move.Moves = this.private_ReadMoves(nodeMove);

					for (var i = 0; i < move.Moves.length; i++)
					{
						move.Moves[i].Parent = move;
					}
				}
				if(this.private_ReadNode(nodeMove, "prisoners") != null || this.private_ReadNodes(nodeMove, "prisoner") != null)
				{
					move.Prisoners = this.private_ReadPrisoners(nodeMove);
				}
				break;
			}
			case "pass" :
			{
				move = new Pass();
				move.PlayerId = parseInt(nodeMove.attributes.getNamedItem("playerId").value);
				if(this.private_ReadNode(nodeMove, "moves") != null || this.private_ReadNodes(nodeMove, "move") != null)
				{
					move.Moves = this.private_ReadMoves(nodeMove);

					for (var i = 0; i < move.Moves.length; i++)
					{
						move.Moves[i].Parent = move;
					}
				}
				if(this.private_ReadNode(nodeMove, "prisoners") != null || this.private_ReadNodes(nodeMove, "prisoner") != null)
				{
					move.Prisoners = this.private_ReadPrisoners(nodeMove);
				}
				break;
			}
			case "capitulation" :
			{
				move = new Capitulation();
				move.PlayerId = parseInt(nodeMove.attributes.getNamedItem("playerId").value);
				break;
			}
		}
	}

	return move;
}

//private
GameConverterXml.prototype.private_ReadPrisoners = function(node){

 var prisoners = null; 
 
 var nodePrisoners = this.private_ReadNode(node, "prisoners"); 
 if(nodePrisoners != null)
 {
  prisoners = new Array();
  var nodesPrisoner = this.private_ReadNodes(nodePrisoners, "prisoner");  
  for (var i = 0; i < nodesPrisoner.length; i++)
  {
   prisoners[prisoners.length] = this.private_ReadPrisoner(nodePrisoners, i);
  }
 }
 else
 {
  if(this.private_ReadNode(node, "prisoner") != null)
  {
   prisoners = new Array();
   prisoners[prisoners.length] = this.private_ReadPrisoner(node, 0);
  }
 }
 
 return prisoners;
}

//private
GameConverterXml.prototype.private_ReadPrisoner = function(node, index){
 
 var prisoner = null; 
 
 if(this.private_ReadNodes(node, "prisoner") != null)
 { 
  prisoner = new Prisoner();
  var nodePrisoner = this.private_ReadNodes(node, "prisoner")[index]; 
  prisoner.PlayerId = parseInt(nodePrisoner.attributes.getNamedItem("playerId").value);
  prisoner.X = parseInt(nodePrisoner.attributes.getNamedItem("x").value);
  prisoner.Y = parseInt(nodePrisoner.attributes.getNamedItem("y").value);
 }
 
 return prisoner;
}
